import pathlib
from typing import Iterator, Union

from shopscraper import json_handler, prod_objs, scraper
from shopscraper.prod_objs import Product


def scrape(
    domain_name: str,
    *,
    include_html: bool = True,
    items_per_page: int = 250,
    max_pages: int = 999,
) -> Iterator[Product]:
    """download products from shopify api and returns product objects

    Args:
        domain_name: fully qualified domain name to webshop (e.g., www.myshopify.com)
        include_html: whether to include body_html field in product objects, default True
        items_per_page: number of items per page. maximum 250, default 250
        max_pages: max number of pages to request, default 999
    Yields:
        ShopifyProduct objects
    """
    n_items = min(items_per_page, 250)
    products = scraper.yield_product_dicts(
        domain_name,
        items_per_page=n_items,
        max_pages=max_pages,
    )
    for product in products:
        yield prod_objs.product_factory(product, include_html)


def read_json(
    file_path: Union[str, pathlib.Path],
    *,
    include_html: bool = True,
) -> Iterator[Product]:
    """creates and yields product objects from json file using write_products_to_json_file function

    Args:
        file_path: path to json file
        include_html: whether to include body_html field in product objects if it exists in json file, default True
    Yields:
        Product objects
    """
    products = json_handler.read_json_file(file_path)
    for product in products:
        yield prod_objs.product_factory(product, include_html)


def scrape_to_json(
    domain_name: str,
    file_path: Union[str, pathlib.Path],
    *,
    include_html: bool = True,
    items_per_page: int = 250,
    max_pages: int = 999,
) -> pathlib.Path:
    """download products from shopify api and write products to json file

    Args:
        domain_name: fully qualified domain name to webshop (e.g., www.myshopify.com)
        file_path: path to file to write products to
        include_html: whether to include body_html field in product objects, default True
        items_per_page: number of items per page, default 250
        max_pages: max number of pages to request, default 999
    Returns:
        pathlib.Path object to json file
    """
    n_items = min(items_per_page, 250)
    products = scraper.yield_product_dicts(
        domain_name,
        items_per_page=n_items,
        max_pages=max_pages,
    )
    file_path = pathlib.Path(file_path)
    json_handler.save_json(products, file_path, include_html)
    return file_path



import json
import pathlib
from typing import Any, Iterator, Union


def _write_first_product_dict(file_path: Union[str, pathlib.Path], product: dict) -> None:
    with open(file_path, "w") as f:
        f.write(f"[{json.dumps(product)}")


def _write_non_first_product_dict(file_path: Union[str, pathlib.Path], product: dict) -> None:
    with open(file_path, "a") as f:
        f.write(f",{json.dumps(product)}")


def _write_final_bracket_to_json_file(file_path: Union[str, pathlib.Path]) -> None:
    with open(file_path, "a") as f:
        f.write("]")


def _manage_html(product: dict, include_html: bool) -> dict:
    if not include_html:
        product = product.copy()
        product["body_html"] = ""
    return product


def _write_individual_products_to_json(
    products: Iterator[dict],
    file_path: Union[str, pathlib.Path],
    include_html: bool,
) -> None:
    """writes product objects to json file

    Args:
        products: iterator of product objects
        file_path: path to json file
        include_html: whether to include body_html field in product objects if it exists in json file, default True
    """
    for i, product in enumerate(products):
        product = _manage_html(product, include_html)
        if i == 0:
            _write_first_product_dict(file_path, product)
        else:
            _write_non_first_product_dict(file_path, product)


def save_json(
    products: Iterator[dict],
    file_path: Union[str, pathlib.Path],
    include_html: bool,
) -> None:
    """writes product objects to json file with correct json formatting

    Args:
        products: iterator of product objects
        file_path: path to json file
        include_html: whether to include body_html field in product objects if it exists in json file, default True
    """
    _write_individual_products_to_json(products, file_path, include_html)
    _write_final_bracket_to_json_file(file_path)


def read_json_file(
    file_path: Union[str, pathlib.Path],
) -> Any:
    """creates and yields product objects from json file using write_products_to_json_file function

    Args:
        file_path: path to json file
    Yields:
        ShopifyProduct objects
    """
    with open(file_path, "r") as f:
        return json.load(f)



import logging
from logging import NullHandler

from shopscraper.api import read_json, scrape, scrape_to_json
from shopscraper.prod_objs import Image, Product, Variant

logging.getLogger(__name__).addHandler(NullHandler())


# def add_stderr_logger(level: int = logging.DEBUG) -> logging.StreamHandler:
#     """
#     Helper for quickly adding a StreamHandler to the logger. Useful for
#     debugging.
#     Returns the handler after adding it.
#     """
#     # This method needs to be in this __init__.py to get the __name__ correct
#     # even if urllib3 is vendored within another package.
#     logger = logging.getLogger(__name__)
#     handler = logging.StreamHandler()
#     handler.setFormatter(logging.Formatter("%(asctime)s %(levelname)s %(message)s"))
#     logger.addHandler(handler)
#     logger.setLevel(level)
#     logger.debug("Added a stderr logging handler to logger: %s", __name__)
#     return handler


# ... Clean up.
del NullHandler
